/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.api.client.GenericType;
import com.sun.jersey.api.client.UniformInterfaceException;
import com.sun.jersey.core.header.MediaTypes;
import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class QuoridorTest extends JerseyTest {
    private File dir;

    public QuoridorTest() throws Exception {
        super("cz.xelfi.quoridor.webidor.resources");
    }

    @Override
    public void setUp() throws Exception {
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        System.setProperty("quoridor.dir", dir.getPath());
        super.setUp();
    }

    @Override
    public void tearDown() throws Exception {
        super.tearDown();
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    /**
     * Test if a WADL document is available at the relative path
     * "application.wadl".
     */
    @Test public void testApplicationWadl() {
        String serviceWadl = webResource.path("application.wadl").
                accept(MediaTypes.WADL).get(String.class);
        assertTrue(serviceWadl.length() > 0);
    }

    @Test public void testCreateAGame() throws Exception {
        Game s = webResource.path("games").queryParam("white", "Jarda")
                .queryParam("black", "Jirka").post(Game.class);

        String msg = webResource.path("games").get(String.class);
        //List<Game> games =  webResource.path("games").get(new GenericType<List<Game>>() {});

        GenericType<List<Game>> gType = new GenericType<List<Game>>() {};

        List<Game> games = webResource.path("games").accept("application/json").get(gType);
        assertEquals("One game", 1, games.size());
        assertEquals("Same white", "Jarda", games.get(0).getWhite());
        assertEquals("Same black", "Jirka", games.get(0).getBlack());

        Game s1 = webResource.path("games/" + s.getId()).queryParam("player", "Jarda").queryParam("move", "N").put(Game.class);
        try {
            Game s2 = webResource.path("games/" + s.getId()).queryParam("player", "Jarda").queryParam("move", "N").put(Game.class);
            fail("Not Jarda's turn, previous call shall fail");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        try {
            Game s2 = webResource.path("games/" + s.getId()).queryParam("player", "Jirka").queryParam("move", "NONSENCE").put(Game.class);
            fail("Invalid move");
        } catch (UniformInterfaceException ex) {
            // OK
        }
        Game s2 = webResource.path("games/" + s.getId()).queryParam("player", "Jirka").queryParam("move", "S").put(Game.class);
        assertNotNull("Successful move", s2);

        File game = new File(new File(dir, "games"), s2.getId());
        assertTrue("File for game exists", game.exists());

        char[] arr = new char[4096];
        FileReader gameContent = new FileReader(game);
        int len = gameContent.read(arr);
        String content = new String(arr, 0, len);

        if (!content.contains("# white: Jarda")) {
            fail(content);
        }
        if (!content.contains("# black: Jirka")) {
            fail(content);
        }
        if (!content.contains("N S")) {
            fail(content);
        }
    }

}
